
# Coin: A SHA2 powered transaction chain with Ed25519 signatures

## Bank

### Request payment

### Send payment

### Send receipts

### Manage accounts and receipts

- Create
- Organize
- Export
- Import

## Chain consensus with PoW

### Authority transaction

    id: 0                                       # Sequential transaction id
    chain: ...sha256...                         # hash of previous transaction in chain
    prev_authority: ...sha256...                # transaction hash of most recent authority transaction
    offset: 0                                   # milliseconds since chain launch
    difficulty: 1                               # 2-byte float from 0 to 2**256
    votes:                                      # `votes` in an authority transaction is a summary
      inflation: 100 * 1024 * 1024              # 100 coin to be generated by each authority transaction
      tax_rate: 1024                            # 1024 units per byte in each transaction is to be destroyed
      auth_period: 60 * 1000                    # 60 seconds between authority transactions
      chain_start: 0                            # Discards UTXOs prior to and including given adjustment tx id
    challenge:
      authority: ...sha256...
      tx1:  ...payment tx...
      tx2: ...payment tx...        
    authority: ...ed25519 key...                # Miner's public key
    nonce: ""                                   # 0-31 bytes of arbitrary data
    
### Payment transaction

    id: 1                                       # Sequential transaction id
    chain: ...sha256...                         # hash of previous transaction in chain
    prev_authority: ...sha256...                # transaction hash of most recent authority transaction
    offset: 0                                   # milliseconds since chain launch
    payment:
      from:
      - tx_hash: ...sha256...
        outs:
        - index: 0
          key: ...ed25519 key...
          signature: ...ed25519 signature...    # Signed by tx[tx_hash].to[index].address
      to:
      - address: ...sha256...                   # Receipients public ed25519 key hashed with SHA256
        amount: 0                               # 1 coin = 1024*1024 units
        memo: ...arbitrary binary data...
      votes:
        inflation: ~                            # Amount of coin generated by authority transactions
        tax_rate: ~                             # Fee per byte of data used by transaction.
        auth_period: ~                          # Target period between authority transactions
        chain_start: ...tx id...                # Discards UTXOs prior to and including given adjustment tx id
    signature:  ...ed25519 signature...         # 64-byte result of ed25519-sign tx by authority

### Consensus

The transaction chain is a linked list which begins with a particular authority transaction for the given chain followed by a sequence of payment transactions and authority transactions.

Authority transactions are generated periodically by random peers performing computational work.  If the hash digest of a valid authority transaction is a member of a relatively small set determined by the work `difficulty` consensus parameter, then the transaction represents a valid proof of work.  The elected `authority` then generates payment transactions, signing any pending payments and appending them to the transaction chain until a new authority transaction is added.  Miners, computational units aimed at generating transactions, should listen for transactions and adjust their own authority transactions accordingly.

To compute the hash digest of an authority transaction, deterministicly represent the transaction object, minus the `nonce` field, as a binary string.  Hash the string with SHA256.  Then, concatenate the intermediate hash digest with the `nonce` and hash again with SHA256.  Difficulty consists of an 8-bit `base` and 8-bit `exp` such that `difficulty == (base + 1) * (2 ** exp) - 1`.  The proof of work is valid if the first 8-bit byte of the final hash digest is numerically less than `base` and is followed by at least `exp` number of zero bits.

Transactions provide a `chain` field, which is the hash digest of the most recent previous transaction, forming a linked list transaction chain.  The transaction also provides a `prev_authority` field, which is the hash digest of the most recent previous authority transaction, forming a second chain that nodes can use to bootstrap their local chain data.  `offset` defines the number of milliseconds elapsed since the first miner began processing the transaction chain and is used to compute `difficulty` adjustments every 4096 blocks.  `difficulty` is adjusted proportionally to the time elapsed since the most recent previous adjustment (`new_difficulty := old_difficulty * (new_elapsed - old_elapsed) / (4096 * auth_period)`).

The `payment` structure defines a set of unspent transaction outputs from previous transactions, and redistributes the total balance, minus transaction data tax, to a new set of outputs.  The `payment` structure also includes a set of votes which determine chain consensus parameters.  



## Network

### UDP Hole punching

### Mesh network with incentivised peering